return function (mod_name)
    local common = {}



    ------------------------------------------------------------------------------------
    -- Get mod name and path to mod
    common.modName = mod_name
    common.modRoot = "__" .. mod_name .. "__"

    ------------------------------------------------------------------------------------
    -- Enable writing to log file until startup options are set, so debugging output
    -- from the start of a game session can be logged. This depends on a locally
    -- installed dummy mod to allow debugging output during development without
    -- spamming real users.
    -- If you don't have this mod or don't want to activate it, you can also change
    -- the value of debug_default!
    local debug_default = false
    if game then
        common.debug_in_log = (game.active_mods["_debug"] and true) or debug_default
    else
        common.debug_in_log = (mods and mods["_debug"] and true) or debug_default
    end

    ------------------------------------------------------------------------------------
    -- Read startup settings
    common.get_startup_settings= function()
        common.dprint("Entered function " .. common.modName .. ".get_startup_settings()")
        local f_name = "common.get_startup_settings()"

        if settings and settings.startup then
            common.long_reach = settings.startup["GCKI-long_reach"].value
            common.debug_in_log = settings.startup["GCKI-debug_in_log"].value
        else
            common.error("Startup settings are not available!", f_name)
        end

        common.dprint("End of function " .. common.modName .. ".get_startup_settings()")
    end

    ------------------------------------------------------------------------------------
    -- Map default Factorio player colors to color names
    common.colors = {
          default = {r = 1.000, g = 0.630, b = 0.259},
          red = {r = 1.000, g = 0.166, b = 0.141},
          green = {r = 0.173, g = 0.824, b = 0.250},
          blue = {r = 0.343, g = 0.683, b = 1.000},
          orange = {r = 1.000, g = 0.630, b = 0.259},
          yellow = {r = 1.000, g = 0.828, b = 0.231},
          pink = {r = 1.000, g = 0.520, b = 0.633},
          purple = {r = 0.821, g = 0.440, b = 0.998},
          white = {r = 0.9, g = 0.9, b = 0.9},
          black = {r = 0.5, g = 0.5, b = 0.5},
          gray = {r = 0.7, g = 0.7, b = 0.7},
          brown = {r = 0.757, g = 0.522, b = 0.371},
          cyan = {r = 0.335, g = 0.918, b = 0.866},
          acid = {r = 0.708, g = 0.996, b = 0.134},
          -- This is not used for settings, it's the default color for
          -- unowned/unlocked vehicles.
          no_color = {r = 0, g = 0, b = 0, a = 0}
    }
    ------------------------------------------------------------------------------------
    -- Read global runtime settings
    common.get_global_settings= function()
        common.dprint("Entered function " .. common.modName .. ".get_global_settings()")
        local f_name = "get_global_settings()"

        if settings and settings.global then
            common.dprint("Read global runtime settings.")
        else
            common.error("Global runtime settings are not available!", f_name)
        end

        common.dprint("End of function " .. common.modName .. ".get_global_settings()")
    end

    ------------------------------------------------------------------------------------
    -- Read per-user runtime settings
    common.get_user_settings = function(player)
        common.dprint("Entered function " .. common.modName .. ".get_user_settings(" ..
                        tostring(player) .. ")")
        local f_name = "get_user_settings()"
        local index = (type(player) == "number") and player or
                      (player and player.valid and player.is_player()) and player.index or
                      GCKI.error("Missing or invalid player " .. tostring(player) .. "!", f_name)

        global.player_settings[index] = {}
        local ps = global.player_settings[index]
--~ common.show (ps, "ps")
        local names = {"debug_in_game", "render_text", "render_time", "render_color", "teleport_to_other_surface"}
        local player = game.players[index]

        -- Read raw settings
        for _, name in ipairs(names) do
            ps[name] = player.mod_settings["GCKI-" .. name].value
        end

        -- Adjust individual settings
        local adjust = {"render_time", "render_color"}
        for _, setting in ipairs(adjust) do
            common.adjust_user_settings(index, setting)
        end
--~ common.show(global.player_settings[index], "global.player_settings[" .. index .. "]")

        common.dprint("End of function " .. common.modName .. ".get_user_settings(" .. tostring(index) .. ")")
    end

    ------------------------------------------------------------------------------------
    -- Change one runtime-per-user setting
    common.get_new_user_setting = function(player, setting)
        common.dprint("Entered function " .. common.modName .. ".get_new_user_setting(" .. tostring(player) ..
                        ", " .. tostring(setting) .. ")")
        local f_name = "get_new_user_setting()"

        local index = (type(player) == "number" and player) or
                      (player and player.valid and player.is_player and player.index) or
                      common.error("Invalid player " .. tostring(player) .. "!", f_name)

        if not (setting and type(setting) == "string") then
            common.error("Invalid setting " .. tostring(setting) .. "!", f_name)
        end

        local var_name = string.gsub(setting, "^GCKI%-", "")

        -- Read raw setting
        global.player_settings[index][var_name] = game.players[index].mod_settings[setting].value
        -- We must convert render_time and render_color to the format used later on
        common.adjust_user_settings(index, var_name)

        common.dprint("End of function " .. common.modName .. ".get_new_user_setting(" .. tostring(player) ..
                        ", " .. tostring(setting) .. ")")
    end

    ------------------------------------------------------------------------------------
    -- Adjust per-user runtime settings
    common.adjust_user_settings = function(player, setting)
        common.dprint("Entered function " .. common.modName .. ".adjust_user_settings(" ..
                        tostring(player) .. ", " .. tostring(setting) .. ")")
        local f_name = "adjust_user_settings()"

        local index = (type(player) == "number" and player) or
                      (player and player.valid and player.is_player and player.index) or
                      common.error("Invalid player " .. tostring(player) .. "!", f_name)

        if not (setting and type(setting) == "string") then
            common.error("Invalid setting " .. tostring(setting) .. "!", f_name)
        end

        local ps = global.player_settings[index]
        -- Adjust render_time
        if setting == "render_time" then
            ps[setting] = ps[setting] * 60
        end
        -- Adjust render_color
        if setting == "render_color" then
            ps[setting] = common.colors[ps[setting]]
        end

        common.dprint("End of function " .. common.modName .. ".adjust_user_settings(" ..
                        tostring(player) .. ", " .. tostring(setting) .. ")")
    end

    ------------------------------------------------------------------------------------
    -- Output: Print to user and debug-print
    common.output = function(player, msg)
        --~ common.dprint("Entered function " .. common.modName .. ".output(" ..
                        --~ tostring(player.name) .. ", " .. tostring({msg}) .. ")")

        if player.valid and player.is_player() then
            player.print({"", msg})
            --~ common.dprint({"", msg})
        elseif not player.valid then
            common.dprint({"", common.modName, ": Couldn't print to player -- player isn't valid!"})
        else
            common.dprint({"", common.modName, ": Couldn't print to " .. player.name .. " -- not a player entity!"})
        end

        --~ common.dprint("End of function " .. common.modName .. ".output(" ..
                        --~ tostring(player.name) .. ", " .. tostring({msg}) .. ")")
    end

    ------------------------------------------------------------------------------------
    -- Debug function
    common.dprint = function(msg)
        --~ log("Entered function " .. common.modName .. ".dprint(" .. tostring({msg}) .. ")")

        if common.debug_in_log then log({"", msg}) end

        if game and msg then
            for _,player in pairs(game.players) do
                local index = player.index
                if global.player_settings and global.player_settings[index]
                                          and global.player_settings[index].debug_in_game then
                    --~ game.players[index].print({"", common.modName, ': ', msg}, {r=.75, g=.5, b=.25})
                    game.players[index].print({"", common.modName, ': ', msg})
                end
            end
        end
        --~ log("End of function " .. common.modName .. ".dprint(" .. tostring({msg}) .. ")")
    end

    --~ ------------------------------------------------------------------------------------
    --~ -- Just a helper that recursively pretty-prints key/value pairs from tables
    --~ -- (See "vararg expression" at <https://www.lua.org/manual/5.2/manual.html#3.4.10>
    --~ --  for the meaning of argument "...")
    --~ common.kv = function(table, ...)
        --~ -- common.dprint("Entered function " .. common.modName .. ".kv(" .. tostring(table) ..
                                    -- ", " .. tostring(... or nil) .. ")")
        --~ local indent = ... or ""

        --~ common.dprint(indent .. "Printing keys/values for table with length " .. tostring(table_size(table)))
        --~ for k,v in pairs(table)  do
            --~ common.dprint({"", indent, 'k: ', tostring(k), '\tv: ', tostring(v)})
            --~ if type(v) == "table" then common.kv(v, indent .. "\t") end
        --~ end

        --~ -- common.dprint("End of function " .. common.modName .. ".kv(" .. tostring(table) ..
                                    -- ", " .. tostring(... or nil) .. ")")
    --~ end

    ------------------------------------------------------------------------------------
    -- Helper function to show value and type of variables
    common.show = function(var, msg)
        -- common.dprint("Entered function " .. common.modName .. ".show(" ..
                        --~ tostring(var) .. ", " .. tostring({msg}) .. ")")

        --~ if type(var) == "table" then
            --~ common.dprint({"", "Show " .. msg .. ":\t"})
            --~ common.kv(var)
        --~ else
            --~ common.dprint({"", "Show " .. msg .. ":\t|" .. tostring(var) .. "| (" .. tostring(type(var)) .. ")"})
        --~ end
        if type(var) == "table" then
            common.dprint({"", "Show", tostring(msg and " " .. msg), ":\t" .. serpent.block(var)})
        else
            common.dprint({"", "Show", tostring(msg and " " .. msg), ":\t|" .. serpent.line(var) .. "|",
                                " (" .. tostring(type(var)) .. ")"})
        end
        -- common.dprint("End of function " .. common.modName .. ".show(" ..
                        --~ tostring(var) .. ", " .. tostring({msg}) .. ")")
    end


    ------------------------------------------------------------------------------------
    -- Output arguments a function was called with
    common.argprint = function(arg)
        --~ local ret
--~ log("arg.valid: " .. serpent.line(arg.valid))
--~ log("type(arg): " .. serpent.line(type(arg)))
--~ log("arg.entity.valid: " .. serpent.line(arg.entity.valid))
--~ log("arg.entity.name: " .. serpent.line(arg.entity.name))

--~ log("type: " .. serpent.line(arg.type))
        -- No argument
        if not arg then
            return "nil"
        -- Argument was player.index or vehicle.unit_number
        elseif type(arg) == "number" then
            return tostring(arg)
        --~ -- Argument is an invalid entity
        --~ elseif not (arg.entity or arg.valid) then
            --~ return "(entity not valid)"
        -- Argument was player entity
        elseif arg.valid and arg.is_player and arg.is_player() then
            return "player " .. tostring(arg.name)
        -- Argument was vehicle entity
        elseif arg.valid and arg.type == "car" then
            return arg.name .. " (" .. arg.unit_number .. ")"
        --~ elseif arg.entity and arg.entity.valid and arg.entity.type == "car" then
        --~ elseif arg.entity then
--~ log("Passed arg.entity")
            --~ if arg.entity.valid then
--~ log("Passed arg.entity.valid")
                --~ if arg.entity.type == "car" then
--~ log("Passed arg.entity.type")
                --~ end
            --~ end
            --~ return arg.entity.name .. " (" .. arg.entity.unit_number .. ")"
        -- Argument was something else
        else
            return "unexpected value " .. serpent.line(arg)
        end
    end


    ------------------------------------------------------------------------------------
    -- Output error messages
    common.error = function(msg, ...)
        -- Optional argument: name of function throwing the error
        local f_name = ...

        error({"", "GCKI error! ", tostring(f_name and f_name .. ": " ), tostring(msg)})
    end


    ------------------------------------------------------------------------------------
    -- Localize entity names for output routines
    -- (Returns localized entity name; falls back to entity name, or to default string)
    common.loc_name = function (entity)
        local f_name = "common.loc_name()"

        --~ if not (entity and entity.valid) then
            --~ common.error("Invalid entity " .. tostring(entity), f_name)
        --~ end

        local unknown_entity = {"GCKI-messages.vehicle_unknown"}

        return entity and entity.valid and entity.localised_name or entity.name or unknown_entity
    end

    ------------------------------------------------------------------------------------
    -- Who's in a vehicle?
    -- car:                                                     get_driver(), get_passenger()
    -- locomotive, artillery-wagon, cargo-wagon, fluid-wagon:   get_driver()
    -- (Returns table with the fields "driver"/"passenger" and the respective player, or nil)
    common.get_occupants = function(vehicle)
        common.dprint("Entered function " .. common.modName .. ".get_occupants(" ..
                        common.argprint(vehicle) .. ")")
        local f_name = common.modName .. ".get_occupants"

        -- Check argument
        if not (vehicle and vehicle.valid and vehicle.type == "car") then
            common.error("Invalid vehicle " .. common.argprint(vehicle), f_name)
        end

        -- Only act on entities of prototype "car"
        local occupants = {}

        local driver = vehicle.get_driver()
        local passenger = vehicle.get_passenger()

        -- Check driver
        if driver then
            -- Driver is player entity
            if driver.is_player() then
                GCKI.dprint("Driver is a player.")
                occupants["driver"] = driver
            -- Driver is player entity that's connected to this character
            elseif driver.player then
                GCKI.dprint("Driver is a character connected to player.")
                occupants["driver"] = driver.player
            -- Driver is an unconnected character (dummy)
            else
                GCKI.dprint("Driver is a dummy character.")
                occupants["driver"] = driver
            end
        end

        -- Check passenger
        if passenger then
            -- Passenger is player entity
            if passenger.is_player() then
                GCKI.dprint("Passenger is a player.")
                occupants["passenger"] = passenger
            -- Passenger is player entity that's connected to this character
            elseif passenger.player then
                GCKI.dprint("Passenger is a character connected to a player.")
                occupants["passenger"] = passenger.player
            -- Driver is an unconnected character (dummy)
            else
                GCKI.dprint("Passenger is a dummy character.")
                occupants["passenger"] = passenger
            end
        end

        common.dprint("End of function " .. common.modName .. ".get_occupants(" ..
                        tostring(vehicle and vehicle.valid and vehicle.name) .. ")")
        return (table_size(occupants) > 0) and occupants or nil
    end

    return common
end
